unit ImgEdt;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TImgEditor = class(TGraphicControl)
  private
    { Private declarations }
    fBitmap    : TBitmap;
    fPixelSize : Word;
    fShowGrid  : Boolean;
    fGridColor : TColor;
    Procedure   SetBitmap (Value : TBitmap);
    Procedure   SetPixelSize (Value : Word);
    Procedure   SetShowGrid (Value : Boolean);
    Procedure   SetGridColor (Value : TColor);
  protected
    { Protected declarations }
    {Override mouse events to convert coordinates to bimap coordinates}
    procedure   MouseDown(Button: TMouseButton; Shift: TShiftState; X: Integer; Y: Integer); override;
    procedure   MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure   MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  public
    { Public declarations }
    Constructor Create (AOwner : TComponent); override;
    Destructor  Destroy; override;

    Procedure   RecalcSize; {Should be protected... see demo-app to find out why it's here}

    Procedure   Paint; override;
  published
    { Published declarations }
    Property Bitmap : TBitmap read fBitmap write SetBitmap;
    Property PixelSize : Word read fPixelSize write SetPixelSize default 5;
    Property ShowGrid : Boolean read fShowGrid write SetShowGrid default True;
    Property GridColor : TColor read fGridCOlor write SetGridColor default clBtnFace;

    Property OnMouseDown;
    Property OnMouseMove;
    Property OnMouseUp;
  end;

procedure Register;

implementation

Constructor TImgEditor.Create (AOwner : TComponent);
Begin
  Inherited Create (AOwner);
  fBitmap := TBitmap.Create;
  fPixelSize := 5;
  fShowGrid := True;
  fGridColor := clBtnFace;
  Width := 100;
  Height := 100;
End;

Destructor  TImgEditor.Destroy;
Begin
  fBitmap.Free;
  Inherited Destroy;
End;

procedure   TImgEditor.MouseDown(Button: TMouseButton; Shift: TShiftState; X: Integer; Y: Integer);
Begin
  Inherited MouseDown (Button, Shift, (X div fPixelSize), (Y div fPixelSize));
End;

procedure   TImgEditor.MouseMove(Shift: TShiftState; X, Y: Integer);
Begin
  Inherited MouseMove (Shift, (X div fPixelSize), (Y div fPixelSize));
End;

procedure   TImgEditor.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
Begin
  Inherited MouseUp (Button, Shift, (X div fPixelSize), (Y div fPixelSize));
End;

Procedure   TImgEditor.SetBitmap (Value : TBitmap);
Begin
  fBitmap.Assign (Value);
  RecalcSize;
End;

Procedure   TImgEditor.SetPixelSize (Value : Word);
Begin
  If Value <> fPixelSize Then Begin
    fPixelSize := Value;
    RecalcSize;
  End;
End;

Procedure   TImgEditor.SetShowGrid (Value : Boolean);
Begin
  If Value <> fShowGrid Then Begin
    fShowGrid := Value;
    Invalidate;
  End;
End;

Procedure   TImgEditor.SetGridColor (Value : TColor);
Begin
  If Value <> fGridColor Then Begin
    fGridColor := Value;
    Invalidate;
  End;
End;

Procedure   TImgEditor.RecalcSize;
Begin
  Width  := fBitmap.Width  * fPixelSize;
  Height := fBitmap.Height * fPixelSize;
End;

Procedure   TImgEditor.Paint;
Var X, Y : Integer;
Begin
{  Inherited Paint; {Doesn't make sense.. it doesn't draw anything...}

  Canvas.Font := Font;
  Canvas.Brush.Color := Color;
  if csDesigning in ComponentState then
    with Canvas do begin
      Pen.Style := psDash;
      Brush.Style := bsClear;
      Rectangle(0, 0, Width, Height);
    end;

  if fBitmap.Empty Then Exit;

  Canvas.StretchDraw (Rect (0, 0, Width, Height), fBitmap);

  If fShowGrid Then Begin
    Canvas.Pen.Color := fGridColor;
    Canvas.Pen.Style := psSolid;
    For Y := 0 to fBitmap.Height-1 Do Begin
      Canvas.MoveTo (0, Y*fPixelSize);
      Canvas.LineTo (Width, Y*fPixelSize);
    End;
    For X := 0 to fBitmap.Width-1 Do Begin
      Canvas.MoveTo (X*fPixelSize, 0);
      Canvas.LineTo (X*fPixelSize, Height);
    End;
  End;
End;

procedure Register;
begin
  RegisterComponents('JocoSoft', [TImgEditor]);
end;

end.
